<?php

class actionIbackuperRestore extends cmsAction {

    public function run() {

        $form = $this->getForm('restore');

        $is_submitted = $this->request->has('submit');

        if ($is_submitted){

            // загружаем файл
            $form_items = $form->parse($this->request, $is_submitted);

            // получаем содержимое файла
            $items = cmsModel::yamlToArray(file_get_contents(cmsConfig::get('upload_path') . $form_items['file_restore']['path']));

            if (!isset($items['ctype'])){
                echo LANG_IBACKUPER_EMPTY_FILE;
                cmsUser::addSessionMessage(LANG_IBACKUPER_EMPTY_FILE, 'error');
                $this->redirectBack();
            }

            // удаляем файл
            cmsCore::getModel('files')->deleteFile($form_items['file_restore']['id']);
            unlink(cmsConfig::get('upload_path') . $form_items['file_restore']['path']);

            if(cmsCore::isControllerExists($items['ctype']['name'])){
                echo LANG_CP_CTYPE_ERROR_NAME;
                cmsUser::addSessionMessage(LANG_IBACKUPER_CTYPE_ISSET, 'error');
                $this->redirectBack();
            }

            // подключаем модели
            $ibackuper_model = cmsCore::getModel('ibackuper');
            $content_model = cmsCore::getModel('content');

            // определяем, есть ли такой тип контента
            $use_prefix = $content_model->getContentTypeByName($form_items['name_prefix']) ? true : false;

            // удаляем id старого типа контента и снимаем блокировку с нового типа контента
            unset($items['ctype']['id']);
            $items['ctype']['is_fixed'] = NULL;

            // если такой тип контента уже есть, используем префикс в названии.
            $items['ctype']['name'] = $use_prefix ? trim($form_items['name_prefix']) . '_' . $items['ctype']['name'] : $form_items['name_prefix'];

            if ($use_prefix){
                // определяем, есть ли такой тип контента
                if ($content_model->getContentTypeByName($items['ctype']['name'])){
                    echo LANG_IBACKUPER_CTYPE_ISSET;
                    cmsUser::addSessionMessage(LANG_IBACKUPER_CTYPE_ISSET, 'error');
                    $this->redirectBack();
                }
            }

            $items['ctype'] = cmsEventsManager::hook('ctype_before_add', $items['ctype']);
            $items['ctype'] = cmsEventsManager::hook("ctype_{$items['ctype']['name']}_before_add", $items['ctype']);

            // добавляем новый тип контента
            $ctype_id = $content_model->addContentType($items['ctype']);
            $items['ctype']['id']=$ctype_id;

            cmsEventsManager::hook('ctype_after_add', $items['ctype']);
            cmsEventsManager::hook("ctype_{$items['ctype']['name']}_after_add", $items['ctype']);

            if ($ctype_id){ cmsCore::getController('content')->addWidgetsPages($items['ctype']); }

            // добавляем категории
            if (!empty($items['cats'])){

                ksort($items['cats']);

                $max_ns_right = 1;

                foreach ($items['cats'] as $key => $value){

                    $max_ns_right = $value['ns_right'] > $max_ns_right ? $value['ns_right'] : $max_ns_right;

                    if ($value['ns_differ'] == NULL){unset($value['ns_differ']);}

                    $content_model->insert('con_' . $items['ctype']['name'] . '_cats', $value);
                }

                $content_model->update('con_' . $items['ctype']['name'] . '_cats', 1, array('ns_right' => $max_ns_right + 1));
            }

            // добавляем поля
            if (!empty($items['fields'])){

                $fields_types = cmsForm::getAvailableFormFields(false);

                foreach ($items['fields'] as $name => $field){
                    $items['fields'][$name]['ctype_id'] = $ctype_id;
                    if (!array_key_exists($field['type'], $fields_types) || !isset($field['type'])){
                        unset($items['fields'][$name]); continue;
                    }
                    unset($items['fields'][$name]['default'], $items['fields'][$name]['rules'], $items['fields'][$name]['handler'], $items['fields'][$name]['handler_title']);
                }

                $ibackuper_model->addCtypeFields($items['ctype']['name'], $items['fields']);
            }

            // добавляем свойства
            if (!empty($items['props'])){

                foreach ($items['props'] as $id => $prop){
                    $prop['ctype_id'] = $ctype_id;
                    if ($prop['options'] == '---'){$prop['options'] = null;}

                    $content_model->insert($content_model->table_prefix . $items['ctype']['name'] . '_props', $prop);

                    cmsEventsManager::hook('ctype_prop_after_add', array($prop, $items['ctype']['name'], $content_model));
                }
            }

            // добавляем связи свойств с категориями
            if ($items['props_bind']){

                foreach ($items['props_bind'] as $id => $prop){
                    $content_model->insert($content_model->table_prefix . $items['ctype']['name'] . '_props_bind', array(
                        'prop_id' => $prop['prop_id'],
                        'cat_id' => $prop['cat_id'],
                        'ordering' => $prop['ordering']
                    ));
                }
            }

            // добавляем наборы
            if (!empty($items['datasets'])){

                foreach ($items['datasets'] as $name => $dataset){

                    unset($dataset['id']);
                    $dataset['id'] = $content_model->addContentDataset($dataset, $items['ctype']);
                }
            }

            // добавляем модераторов
            if ($items['moderators']){

                $users_model = cmsCore::getModel('users');

                foreach ($items['moderators'] as $id => $moderator){

                    $users_model->filterEqual('nickname', $moderator['user_nickname']);
                    $users_model->filterEqual('id', $moderator['user_id']);
                    $users_ids = $users_model->getUsersIds();

                    if(!empty($users_ids)){
                        $ibackuper_model->insert('moderators', array(
                            'ctype_name' => $items['ctype']['name'],
                            'user_id' => $moderator['user_id'],
                            'date_assigned' => ''
                        ));
                    }
                }
            }

            // добавляем доступы
            if (!empty($items['perm_values'])){
                cmsPermissions::savePermissions($items['ctype']['name'], $items['perm_values']);
            }

            // добавляем настройки создания карты сайта
            if (!empty($items['sitemap_options'])){
                $sitemap_options = cmsController::loadOptions('sitemap');
                $sitemap_options['sources']['[content|' . $items['ctype']['name'] . ']'] = 1;
                cmsController::saveOptions('sitemap', $sitemap_options);
            }

            // добавляем пресеты изображений, если их нет
            if (!empty($items['images_presets'])){

                $presets = $this->model->get('images_presets', function($item, $model){
                    return $item['title'];
                }, 'name');

                if ($presets){
                    foreach ($presets as $name => $title) {
                        if (!empty($items['images_presets'][$name])){ unset($items['images_presets'][$name]); }
                    }
                    if (!empty($items['images_presets'])){
                        foreach ($items['images_presets'] as $name => $value){
                            $this->model->insert('images_presets', $value);
                        }
                    }
                }
            }

            cmsUser::addSessionMessage(sprintf(LANG_IBACKUPER_CTYPE_ADD_SUCCESS, $items['ctype']['name']), 'success');
            $this->redirectBack();
        }

        // выводим форму
        return cmsTemplate::getInstance()->render('backend/restore', array(
            'form' => $form
        ));
    }
}